// __multiversion__
// This signals the loading code to prepend either #version 140 or #version 330 es as apropriate.
//#extension EXT_color_buffer_float : enable
//#extension EXT_sparse_texture : enable
//#extension EXT_texture_format_BGRA8888 : enable
//#extension EXT_texture_sRGB_decode : enable
//#extension EXT_clear_texture : enable

#include "fragmentVersionCentroid.h"
#include "debug/definitions.h"
#include "debug/alpha_features.txt"
#include "settings.h"
#define Libra_Equations gl_FragColor
#define sampler_tex diffuse
#define c cos
#define sq sqrt

//--------------------------------------------------------------------------
//Libra Shader, created by JexCrafter do not modify or rewrite the codes without my permission sharing to public should credit me//

//(L I B R A  -  S H A D E R)®//

//--------------------------------------------------------------------------

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;
varying highp vec3 position;
varying highp vec3 wpos;
varying highp vec3 p;

#ifdef FOG
varying vec4 fogColor;
#endif

#include "uniformPerFrameConstants.h"
#include "uniformPerFrameConstants.h"
#include "uniformShaderConstants.h"
#include ".tmp"
#include "util.h"

bool isReflective(vec4 tex, vec4 inColor) {
    return 0.95 < tex.a && tex.a < 1.0 && (inColor.b == inColor.g && inColor.r == inColor.g);
}

//Noise 2D Basic
highp float hash(highp float p) { p = fract(p * 0.011); p *= p + 7.5; p *= p + p; return fract(p); }
highp float hash(highp vec2 p) {highp vec3 p3 = fract(vec3(p.xyx) * 0.22); p3 += dot(p3, p3.yzx + 3.333); return sqrt(fract((p3.x + p3.y) * p3.z)); }

highp float noise(highp float x) {
    highp float i = floor(x);
    highp float f = fract(x);
    highp float u = f * f * (3.0 - 2.0 * f);
    return sqrt(mix(hash(i), hash(i + 1.0), u));
}

highp float noise(highp vec2 x) {
    highp vec2 i = floor(x);
    highp vec2 f = fract(x);

	highp float a = hash(i);
    highp float b = hash(i + vec2(1.0, 0.0));
    highp float c = hash(i + vec2(0.0, 1.0));
    highp float d = hash(i + vec2(1.0, 1.0));
    
    highp vec2 u = f * f * (3.0 - 2.0 * f);
	return sqrt(mix(a, b, u.x) + (c - a) * u.y * (1.0 - u.x) + (d - b) * u.x * u.y);
}

highp float noise(highp vec3 x) {
    const highp vec3 step = vec3(110, 241, 171);

    highp vec3 i = floor(x);
    highp vec3 f = fract(x);
 
    highp float n = dot(i, step);

    highp vec3 u = f * f * (3.0 - 2.0 * f);
    return mix(mix(mix( hash(n + dot(step, vec3(0, 0, 0))), hash(n + dot(step, vec3(1, 0, 0))), u.x),
                   mix( hash(n + dot(step, vec3(0, 1, 0))), hash(n + dot(step, vec3(1, 1, 0))), u.x), u.y),
               mix(mix( hash(n + dot(step, vec3(0, 0, 1))), hash(n + dot(step, vec3(1, 0, 1))), u.x),
                   mix( hash(n + dot(step, vec3(0, 1, 1))), hash(n + dot(step, vec3(1, 1, 1))), u.x), u.y), u.z);
}

  #ifdef FILM_GRAIN

	float rand(vec2 coord) {
	  return fract(sin(dot(coord.xy, vec2(12.9898, 78.233))) * 43758.5453);
	}

	vec3 filmgrain(vec3 color) {

		const float noiseAmount = 0.01;

		vec2 coord = gl_FragCoord.xy + TIME * 0.01;

		vec3 noise = vec3(1.0f);
				 noise.r = rand(coord + 0.1);
				 noise.g = rand(coord);
				 noise.b = rand(coord - 0.1);

		return color * (0.9999 - tan(noiseAmount) + tan(noise) * noiseAmount) + noise * noiseAmount;

	}
  #endif

float luma(vec3 clr) {
	return dot(clr, vec3(0.33333)-vec3(0.2125, 0.7154, 0.0721)*0.4 );
}

#define TONEMAP //Final Base
highp float v3x(const float x) {
highp float A = 5.5;
highp float B = 0.05;
highp float C = 3.55;
highp float D = 3.85;
highp float E = 0.05*2.0;
highp float F = 0.35;
return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

vec3 GOUTTE_ECRAN_1(vec3 x) { 
mediump float a=.015;
mediump float b=.005;
mediump float c=.03;
mediump float d=.003;
mediump float e=15.;
mediump float f=1.2; 
return ((x*(a*x+c*b)+d*e)/(x*(a*x+b)+d*f))-e/f; 
}
vec3 GOUTTE_ECRAN_2(vec3 x) {
mediump float a=.07;
mediump float b=.03;
mediump float c=.1;
mediump float d=.03;
mediump float e=15.;
mediump float f=1.2; 
return ((x*(a*x+c*b)+d*e)/(x*(a*x+b)+d*f))-e/f; 
}

vec3 Filmic(vec3 clr) {
highp float W = 1.3 / 1.0; //inverse value
#ifdef TONEMAP
float Luma = dot(clr, sin(POSTPROCESSING));
vec3 Chroma = clr - Luma;
clr = (Chroma * 1.70) + Luma;
clr = vec3(v3x(clr.r), v3x(clr.g), v3x(clr.b)) / v3x(W);
#endif
return clr;
}

vec3 uncharted2_tonemap_partial(vec3 x) { 
highp float A = 0.15f; 
highp float B = 0.50f; 
highp float C = 0.10f; 
highp float D = 0.20f; 
highp float E = 0.02f; 
highp float F = 0.30f; 
return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F; 
} 

vec3 uncharted2_filmic(vec3 v) { 
float exposure_bias = 2.0f; 
vec3 curr = uncharted2_tonemap_partial(v * exposure_bias); 
vec3 W = vec3(11.2f); 
const vec3 luminace = tan(vec3(0.2125, 0.7154, 0.0721));
vec3 white_scale = vec3(1.0f) / uncharted2_tonemap_partial(W+luminace); 
return curr * white_scale;
}

vec3 LinearToGamma(vec3 c)
{
	return pow(c, vec3(1.0*Colored_Gamma / 2.2));
}

vec3 GammaToLinear(vec3 c)
{
	return pow(c, vec3(2.2)*.962);
}

vec3 sqrt1vec3(vec3 vector) {
    vector = vec3(1.0) - vector;
    vector *= vector;
    return vec3(1.0) - vector;
}

 /**
 * Adjusts the saturation of a color.
 * 
 * @name czm_saturation
 * @glslFunction
 * 
 * @param {vec3} rgb The color.
 * @param {float} adjustment The amount to adjust the saturation of the color.
 *
 * @returns {float} The color with the saturation adjusted.
 *
 * @example
 * vec3 greyScale = czm_saturation(color, 0.0);
 * vec3 doubleSaturation = czm_saturation(color, 2.0);
 */
vec3 czm_saturation(vec3 rgb, float adjustment)
{
    // Algorithm from Chapter 16 of OpenGL Shading Language
    const vec3 W = vec3(0.2125, 0.7154, 0.0721);
    vec3 intensity = vec3(dot(rgb, W));
    return mix(intensity, rgb, adjustment);
}

highp float bloom(vec2 pos){
highp float blur;
blur = clamp(1.-length(pos)*.1,.0,1.);
return blur;}

// http://http.developer.nvidia.com/GPUGems3/gpugems3_ch24.html
vec3 linear_to_srgb( vec3 color ) {
	const float p = 1. / 1.2;
	return vec3(pow(color.r, p), pow(color.g, p), pow(color.b, p));
}

vec3 xyY_2_XYZ( vec3 xyY)
{
	vec3 XYZ;
	XYZ[0] = xyY[0] * xyY[2] / max( xyY[1], 1e-10);
	XYZ[1] = xyY[2];  
	XYZ[2] = (1.0 - xyY[0] - xyY[1]) * xyY[2] / max(xyY[1], 1e-10);

	return XYZ;
}

highp float voronoi(highp vec2 p){
    highp vec2 cell_i = floor(p);
    highp vec2 cell_f = fract(p);
    highp float min_dist = 2.0;
    for(int x = 0; x <= 1; x++){
    for(int y = 0; y <= 1; y++){
 	 highp vec2 xy = vec2(float(x), float(y));
 	 highp float n = dot(cell_i+xy, vec2(1,57));
    highp vec2 point = vec2(hash(n));
    point = 0.3 * sin(
    TIME*0.125 + 6.2831 * point);
    highp float dist = distance(
     	     cell_f, xy + point);
    min_dist = min(min_dist, dist);
        }
    }
	return min_dist;
}

// traditional fractal brownian for reflect water
highp float fbm(highp vec2 p){
highp float f = 0.0;
	f += 0.25*noise( p ); p = p*2.03;
	f += 0.12*noise( p ); p = p*2.04;
return f*0.7;
}

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else

float rf = isReflective(texture2D(TEXTURE_0, uv0), color) ? 0.0 : 0.0;

#if USE_TEXEL_AA
	vec4 diffuse = bool(rf) ? vec4(vec3(1)*0.1, texture2D_AA(TEXTURE_0, uv0).a) : texture2D_AA(TEXTURE_0, uv0);
#else
	vec4 diffuse = bool(rf) ? vec4(vec3(1)*0.1, texture2D(TEXTURE_0, uv0).a) : texture2D(TEXTURE_0, uv0);
#endif
	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif
	
vec4 inColor = tan(color)*1.1;
vec3 shd = vec3(0.94)*vec3(1.0)*1.2;
highp float traced_no_lights = (1.-uv1.x);
highp float darkness = smoothstep(1.0, 0.5, uv1.y);

highp float shadow = smoothstep(0.876,0.869,uv1.y);
shadow *= traced_no_lights;
shadow *= mix(0.0,0.2,shadow); //minus value
diffuse.rgb = mix(diffuse.rgb,vec3(0.2), shadow);

highp float uY = uv1.y; //trees shadows accidentally removed from previous updates.

//highp float detect_rain = smoothstep( .855, .875, uv1.y );

lowp vec3 block = normalize(cross(dFdx(position),dFdy(position)))*0.700;
float drilight = max(0.0,max(block.z,-block.z));
diffuse.rgb *= mix(1.1,0.46, drilight)*0.770; //we need to multiply drilight because it's modifies the light and the darkness from shader, multiply based on shader needs
//-darker
//+brighter

//@Credits to ; @Lebrino Hermie 
//from drilight code

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

//nColor
vec4 tex1 = texture2D( TEXTURE_1, uv1 );
vec3 ntime = vec3(0.94);

tex1.rgb *= sin(vec3(1.0))+uv1.x*1.1/1.0;

#ifndef ALWAYS_LIT
	diffuse *= tex1;

  #ifdef Sphere_On_Water
highp vec4 sphere = vec4(1.2, 1.2, 1.2, 1.0) * 0.7;
highp float reflect_power = 1990.0;
highp vec3 light_pos = vec3(10000, 6000, 180);
highp vec3 P = wpos;
highp vec3 N = normalize(wpos);
highp vec3 V = normalize(-wpos);
highp vec3 L = normalize(sqrt(light_pos - P));
highp vec3 R = reflect(normalize(-L), N);
highp vec4 reflect = pow(max(0.0, dot(R, V)), reflect_power) * sphere * 1000.0;

  #endif
#endif

#ifndef SEASONS
	#if USE_ALPHA_TEST
		diffuse.a = inColor.a; //leaves
  #ifdef Leaves_Color_g_Blooms
	//diffuse.rgb += tan(0.0099);
	//diffuse.rgb += tan(0.0099);
	//diffuse.rgb += tan(0.0099);
  #endif
	#endif
	
  diffuse.rgb *= sqrt(inColor.rgb)*0.887;
	diffuse.rgb += tan(diffuse.rgb)*0.01;
	diffuse.rgb += tan(diffuse.rgb)*0.01;
	diffuse.rgb += tan(diffuse.rgb)*0.01;
#else
	vec2 uv = inColor.xy*1.0;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_1, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	//diffuse.a = 0.0; //no alpha
#endif

//boolean definitions
bool nether = (FOG_COLOR.r>FOG_COLOR.b && FOG_COLOR.r<0.5 && FOG_COLOR.b<0.2);
bool unwater = (color.b>color.r);
bool water = (color.b > color.r || rf > 0.0);

#include "shader_assets/terrain_gbuffer.fsh"
#include "shader_assets/water_gbuffer.fsh"
#include "shader_assets/dataRGB.glsl"

diffuse.rgb += vec3(-0.11111111);
  #ifdef Sun_Blooms
if(nether==false){
if(FOG_COLOR.r*1.0>FOG_COLOR.b){
	
vec3 bnm = noise(vec2(TIME *0.4+wpos.z/wpos.y))*vec3(1.3,0.7,0.17) * 0.400;

vec3 sb = mix(vec3(0.0),bnm,clamp(length(wpos)/RENDER_DISTANCE*0.8,0.0,1.0));

diffuse.rgb += bloom(wpos.yz-vec2(6.,.0))*0.056;

diffuse.rgb += sb;

}}
  #endif
  #ifdef Nether_Fogs
if(nether == true){
diffuse.rgb += diffuse.rgb * (vec3(0.30, 0.30, 0.30) * 1.5) * pow(uv1.x * 1.20,5.3);
}
  #endif
  #ifdef Secondary_Fogs
vec3 fog = mix(diffuse.rgb, FOG_COLOR.rgb * vec3(vec4(1.0)), clamp(length(-wpos * 0.5)/RENDER_DISTANCE * 1.0, 0.0, 1.0));
diffuse.rgb = fog * MATH_PI;
  #endif
//float sampler_tex = texture2D(TEXTURE_1, vec2(0.0, 1.0)).r;
//diffuse.rgb = sampler_tex;

diffuse.rgb += vec3(is_light)*0.1 / skylight *0.712 * uv1.x;

//if(unwater==false){
//float is_wet = sin(position.x+position.y+position.z)*0.045;
//sampler.rgb += is_wet * detect_rain * uv1.y;
//sampler.rgb *= 0.900;}

diffuse.rgb = fract(linear_to_srgb(diffuse.rgb)*1.010)*001.0;

//vec3 lightSources = vec3(255,240,180);

//diffuse.rgb = czm_luminance(diffuse.rgb);

diffuse.rgb -= vec3(1.0)*fract(001.000);
diffuse.rgb -= vec3(1.0)*fract(001.000);
diffuse.rgb -= vec3(1.0)*fract(001.000);
//limit of light's

diffuse.rgb += uv1.x*0.00;//dont use uv
//diffuse.rgb += vec3(0.2125, 0.7154, 0.0721)*0.001;


//diffuse.rgb += uv1.x*0.0670;
	//diffuse.rgb += sqrt(color.g) * 0.02 * color.b;

if(color.r < color.b){
diffuse.rgb *= 1.10;
diffuse.rgb *= diffuse.rgb; //double resolution
}

if(color.a < color.g){
diffuse.rgb += 0.002; //trees and alpha textures and can affect birch trees leaves
//use to lift brightness on birch leaves
//not sure what textures affected but this value are already normalized
}

#define Use_Normal_Exposure

  #ifdef Use_Normal_Exposure
if(color.a == color.g){
diffuse.rgb *= 0.779 + vec3(1.0) * 00.16; //detect block's with gamma & exposure's
}
if(color.g < color.a){
diffuse.rgb *= 0.779 + vec3(1.0) * 00.16; //detect block's with gamma & exposure's
}
//calculate outside and top textures detection for exposed textures overlap on uv's and fully mask the exposed textures like sand, gravel , etc.
  #endif

if(color.r <= color.g){
diffuse.rgb += 0.002 * 0.0004572 ;
//diffuse.rgb *= diffuse.rgb; //top of blocks
}

if(color.g == 0.0){
diffuse.rgb += GOUTTE_ECRAN_1(diffuse.rgb)*1.0;
diffuse.rgb += GOUTTE_ECRAN_2(diffuse.rgb)*1.0;
diffuse *= 1.0;
}

//if(uv1.x < 189.0){
//diffuse.rgb += uv1.x*vec3(1.0)*0.400;
//}
//diffuse += uv1.x*0.0861518965;

vec3 rgb_correction = (sin(diffuse.rgb*vec3(1.0)*0.200)*0.200+vec3(1.0)*00.1)*0.400;

	diffuse.rgb += rgb_correction * darkness;
diffuse.rgb += vec3(vec4(1.0))*0.001;
diffuse.rgb += vec3(vec4(1.0))*0.002;
diffuse.rgb += vec3(vec4(1.0))*0.002;
diffuse.rgb += vec3(vec4(0.94))*0.0027832;
diffuse.rgb += vec3(vec4(0.94))*0.0007832;
diffuse.rgb += vec3(vec4(0.94))*0.0007832;

diffuse.rgb += vec3(vec4(1.0))*0.00100;

  #ifdef Leaves_Color_g_Blooms //bloom range
if(color.a == 0.0){
diffuse.rgb += xyY_2_XYZ(diffuse.rgb)*0.002+0.002+0.001;
}
  #endif

#ifdef FOG
if(FOG_CONTROL.x == 0.0){
#include "shader_assets/unwater_gbuffer.fsh"
}
#endif

#ifdef FOG
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a );
#endif

//-----------------------------------------------------------------------//
vec4 RESULT = l
*l
  *fl 
 *l*
fl / 
(sqrt(vec4(1.0f)*1.)


*m1 *m2)

*m3
*1.0
*m0   
*1.0 *vec4(1.0,1.0,1.0,1.0)*
vec4(1.0)*
vec4(1.0)*
1.01000
+FOG_COLOR*
vec4(1.0) 
*0.1*l*1.0
*sqrt(l*l*fl)*normalize(vec4(1.0));
#define PSDensity 1.0855159
#define quality_speed 0.0066183 + 0.94 * 0.00236
vec4 RESOLUTION_0 = texture2D(TEXTURE_0, uv0) * quality_speed;
vec4 RESOLUTION_1 = texture2D(TEXTURE_1, uv1) * quality_speed;


//-----------------------------------------------------------------------//



	Libra_Equations = RESULT * PSDensity + RESOLUTION_0 * RESOLUTION_1;










#endif // BYPASS_PIXEL_SHADER







}
